/*
 * @Descripttion: 
 * @version: 
 * @Author: penghui
 * @Date: 2023-07-12 15:12:35
 * @LastEditors: penghui
 * @LastEditTime: 2023-07-13 20:45:43
 */
import { Component, Enum, Graphics, Material, Size, Vec2, _decorator, v2, view ,Node, EventTouch, v3,screen, UITransform, Vec3, Mask} from "cc";
import { Bound } from "./QuadTree";
import { Intersection } from "./LightStruct";

const {ccclass, property} = _decorator;

export enum LightType {
    Round,      // 圆形
    Sector,     // 扇形
}

let LightBound = new Bound(0, 0, 0, 0);
@ccclass
export default class Light extends Component {
    /** 画笔 */
    @property(Graphics)
    graphics: Graphics = null;
    @property({type: Enum(LightType)})
    lightType: LightType = 0;

    @property(Graphics)
    maskgraphics: Graphics = null;

    canvasSize: Size = null;
    visiableSize: Size = null;

    radius = 200;
    fade = 1;

    // graphics
    private _material: Material = null;

    onLoad () {
        this._material = this.graphics.customMaterial;
    
        this.canvasSize = screen.windowSize;
        this.visiableSize = screen.windowSize;
        this.node.getComponent(UITransform).setContentSize(this.canvasSize);

        // return
        this._material.setProperty("screen", v2(this.canvasSize.width, this.canvasSize.height));


        let r = this.radius / this.canvasSize.width;
        this._material.setProperty("maxRadius", r);
    }

    
    start () {
        this.node.on(Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
    }

    onDestroy() {
        this.node.off(Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
    }

    private onTouchMove(ev: EventTouch) {
        let v2pos =  new Vec3(ev.getUILocation().x,ev.getUILocation().y,0);
        let nodepos = this.node.parent.getComponent(UITransform).convertToNodeSpaceAR(v2pos);
        
        this.node.setPosition(nodepos);
    }

    public getBound() {
        let pos = this.node.getPosition();
        LightBound.x = pos.x-this.radius/2; LightBound.y = pos.y-this.radius/2; 
        LightBound.width = LightBound.height = this.radius;
        return LightBound;
    }
    

    /** 绘制光 */
    public draw(intersections: Intersection[]) {
        let lightPos = v2(this.node.getPosition().x,this.node.getPosition().y);
        this._doDraw(this.graphics, lightPos, intersections);
        this._material.setProperty("lightPos", v2(lightPos.x/this.visiableSize.width, lightPos.y/this.visiableSize.height));
    }

    /** 绘制light */
    private _doDraw(graphics: Graphics, light: Vec2, intersects: Intersection[]) {
        
        graphics.clear();
        graphics.moveTo(intersects[0].x, intersects[0].y);
        for(let i=1; i<intersects.length; i++) {
            let intersect = intersects[i];
            graphics.lineTo(intersect.x, intersect.y);
        }
        graphics.moveTo(intersects[0].x, intersects[0].y);
        
        graphics.fill();

        // this.maskgraphics.clear();
        // this.maskgraphics.moveTo(intersects[0].x, intersects[0].y);
        // for(let i=1; i<intersects.length; i++) {
        //     let intersect = intersects[i];
        //     this.maskgraphics.lineTo(intersect.x, intersect.y);
        // }
        // this.maskgraphics.moveTo(intersects[0].x, intersects[0].y);
        
        // this.maskgraphics.fill();

        // for(let i=0; i<intersects.length; i++) {
        //     let intersect = intersects[i];
        //     graphics.moveTo(light.x, light.y);
        //     graphics.lineTo(intersect.x, intersect.y);
        //     graphics.stroke();
        // }
    }
        
    // update (dt) {}
}