import { _decorator, Component, Vec2, js, view } from "cc";
import Quadtree, { Bound } from "./QuadTree";


const {ccclass, property} = _decorator;

@ccclass
export default class Obstacle extends Component {

    private _quadtree: Quadtree = null;
    private _polygons: {[key: string]: Vec2[]} = js.createMap();


    onLoad() {
        let size = view.getVisibleSize();
        this._quadtree = new Quadtree({x: 0, y: 0, width: size.width, height: size.height});
    }

    public addPolygon(uid: string, points: Vec2[]) {
        this._polygons[uid] = points;
        let bound = this._getBound(points);
        bound.uid = uid;
        this._quadtree.insert(bound);
    }

    public removePolygon(uid: string) {
        // todo...
    }

    public getPolygons(bound: Bound) {
        let bounds = this._quadtree.retrieve(bound);
        let polygons: Vec2[][] = [];
        for(const e of bounds) {
            polygons.push(this._polygons[e.uid]);
        }
        return polygons;
    }

    private _getBound(points: Vec2[]) {
        let point = points[0];
        let xMin = point.x;
        let xMax = point.x;
        let yMin = point.y;
        let yMax = point.y;
        for(let i=0; i<points.length; i++) {
            if(points[i].x < xMin) xMin = points[i].x;
            if(points[i].x > xMax) xMax = points[i].x;
            if(points[i].y < yMin) yMin = points[i].y;
            if(points[i].y > yMax) yMax = points[i].y;
        }
        return new Bound(xMin, yMin, xMax - xMin, yMax - yMin);
    }

    public clear() {
        this._quadtree.clear();
        this._polygons = {};
    }

}